import contextlib
import random
import sys
import test.support
import types
import unittest

from jaraco.test.cpython import from_test_support, try_import


warnings_helper = try_import('warnings_helper') or from_test_support('check_warnings')


class support_compat:
    if sys.version_info < (3, 9):
        def requires_zlib(reason='requires zlib'):
            try:
                import zlib
            except ImportError:
                zlib = None
            return unittest.skipUnless(zlib, reason)

        def requires_gzip(reason='requires gzip'):
            try:
                import gzip
            except ImportError:
                gzip = None
            return unittest.skipUnless(gzip, reason)

        def requires_bz2(reason='requires bz2'):
            try:
                import bz2
            except ImportError:
                bz2 = None
            return unittest.skipUnless(bz2, reason)

        def requires_lzma(reason='requires lzma'):
            try:
                import lzma
            except ImportError:
                lzma = None
            return unittest.skipUnless(lzma, reason)


support = types.SimpleNamespace(**{**vars(test.support), **vars(support_compat)})


class RandomCompat(random.Random):
    def randbytes(self, n):
        """Generate n random bytes."""
        return self.getrandbits(n * 8).to_bytes(n, 'little')


Random = RandomCompat if sys.version_info < (3, 9) else random.Random


if sys.version_info < (3, 9):

    def removesuffix(self, suffix):
        # suffix='' should not call self[:-0].
        if suffix and self.endswith(suffix):
            return self[: -len(suffix)]
        else:
            return self[:]

    def removeprefix(self, prefix):
        if self.startswith(prefix):
            return self[len(prefix) :]
        else:
            return self[:]
else:

    def removesuffix(self, suffix):
        return self.removesuffix(suffix)

    def removeprefix(self, prefix):
        return self.removeprefix(prefix)


@contextlib.contextmanager
def temp_tarfile_open(DIR, tarname):
    """
    Syntax compatibility for Python 3.8 for:

    ```
    with (
        os_helper.temp_dir(DIR),
        tarfile.open(tarname, encoding="iso8859-1") as tar
    ):
    ```
    """
    from .py310 import os_helper
    import tarfile
    with os_helper.temp_dir(DIR), tarfile.open(tarname, encoding="iso8859-1") as tar:
        yield tar



import contextlib
import sys
from .py39 import support as std_support
from .py39 import os_helper as std_os_helper
import types
import warnings as std_warnings



try:
    from test import archiver_tests
except ImportError:
    from . import archiver_tests  # noqa: F401


class os_helper_compat:
    def skip_unless_working_chmod(test):
        """Never skip"""
        return test

    def can_chmod():
        return True


os_helper = types.SimpleNamespace(**{**vars(os_helper_compat), **vars(std_os_helper)})


class support_compat:
    def is_emscripten():
        return False

    def is_wasi():
        return False


support = types.SimpleNamespace(**{**vars(support_compat), **vars(std_support)})


class warnings_compat:
    if sys.version_info < (3, 11):
        @contextlib.contextmanager
        def catch_warnings(*, record=False, module=None, action=None, **kwargs):
            with std_warnings.catch_warnings(record=record, module=module) as val:
                if action:
                    std_warnings.simplefilter(action, **kwargs)
                yield val

warnings = types.SimpleNamespace(**{**vars(std_warnings), **vars(warnings_compat)})



import pathlib
import sys
from test import support


def find_file(name, subdir=None):
    return str(pathlib.Path(*filter(None, ('tests', subdir, name))).absolute())


def patch_findfile():
    """
    Early hook to ensure findfile behaves differently before test_tarfile is imported.
    """
    support.findfile = find_file


def backport_as_std():
    """
    Make sure 'import tarfile' gets the backport.
    """
    from backports import tarfile
    sys.modules['tarfile'] = tarfile


def patch_bz2():
    """
    Ensure bz2.BZ2File reflects the name.
    """
    if sys.version_info > (3, 13):
        return
    try:
        import bz2
    except ImportError:
        return

    bz2.BZ2File.name = property(lambda self: self._fp.name)


def patch_lzma():
    """
    Ensure lzma.LZMAFile reflects the name.
    """
    if sys.version_info > (3, 13):
        return
    try:
        import lzma
    except ImportError:
        return

    lzma.LZMAFile.name = property(lambda self: self._fp.name)


def pytest_configure():
    patch_findfile()
    backport_as_std()
    patch_bz2()
    patch_lzma()



"""
Backward-compatability shims to support Python 3.9 and earlier.
"""

import sys
from .py38 import support as std_support
import types

from jaraco.test.cpython import from_test_support, try_import

os_helper = try_import('os_helper') or from_test_support(
    'TESTFN',
    'temp_cwd',
    'skip_unless_symlink',
    'unlink',
    'rmtree',
    'temp_dir',
    'change_cwd',
    'create_empty_file',
    'rmdir',
)


class FakePath:
    """Simple implementation of the path protocol.
    """
    def __init__(self, path):
        self.path = path

    def __repr__(self):
        return f'<FakePath {self.path!r}>'

    def __fspath__(self):
        if (isinstance(self.path, BaseException) or
            isinstance(self.path, type) and
                issubclass(self.path, BaseException)):
            raise self.path
        else:
            return self.path


os_helper.FakePath = getattr(os_helper, 'FakePath', FakePath)


class support_compat:
    if sys.version_info < (3, 10):
        def check__all__(test_case, mod, *, not_exported=(), **kwargs):
            kwargs.update(blacklist=not_exported)
            return std_support.check__all__(test_case, mod, **kwargs)


support = types.SimpleNamespace(**{**vars(std_support), **vars(support_compat)})
